﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Info2Praktikum;

namespace _2011_05_04_Praktikum
{
    public abstract class Filter
    {
        public abstract Bitmap Execute(Bitmap input);
        public virtual void ShowSettings()
        {
            // standardmäßig keinen Dialog zeigen
        }
        public virtual void CloseSettings()
        {
        }
    }

    public class Inverter : Filter
    {
        public override Bitmap Execute(Bitmap input)
        {
            Bitmap result = new Bitmap(input.Width, input.Height);
            for (int y = 0; y < result.Height; y++)
            {
                for (int x = 0; x < result.Width; x++)
                {
                    result.SetRed(x, y, 255 - input.GetRed(x, y));
                    result.SetGreen(x, y, 255 - input.GetGreen(x, y));
                    result.SetBlue(x, y, 255 - input.GetBlue(x, y));
                }
            }
            return result;
        }

        public override string ToString()
        {
            return "Farben umkehren";
        }
    }

    public class HorizontalFlipper : Filter
    {
        public override Bitmap Execute(Bitmap input)
        {
            Bitmap result = new Bitmap(input.Width, input.Height);
            for (int y = 0; y < result.Height; y++)
            {
                for (int x = 0; x < result.Width; x++)
                {
                    int x1 = result.Width - 1 - x;
                    result.SetRed(x1, y, input.GetRed(x, y));
                    result.SetGreen(x1, y, input.GetGreen(x, y));
                    result.SetBlue(x1, y, input.GetBlue(x, y));
                }
            }
            return result;
        }

        public override string ToString()
        {
            return "Horizontal spiegeln";
        }
    }

    public class Cropper : Filter
    {
        int x, y, width, height;

        public Cropper(int x, int y, int width, int height)
        {
            this.x = x;
            this.y = y;
            this.width = width;
            this.height = height;
        }

        public override Bitmap Execute(Bitmap input)
        {
            Bitmap result = new Bitmap(width, height);
            for (int y1 = 0; y1 < height; y1++)
            {
                // Math.Min, damit nicht ggf. über Rand hinaus
                int y2 = Math.Min(y1, input.Height-1);
                for (int x1 = 0; x1 < width; x1++)
                {
                    int x2 = Math.Min(x1, input.Width-1);
                    result.SetRed(x1, y1, input.GetRed(x2, y2));
                    result.SetGreen(x1, y1, input.GetGreen(x2, y2));
                    result.SetBlue(x1, y1, input.GetBlue(x2, y2));
                }
            }
            return result;
        }

        public override string ToString()
        {
            return "Bildausschnitt " + x +", " + y + ", " + (x+width) + ", " + (y+height);
        }
    }
}
